/*
 * Copyright (c) 2009-2011 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.app;

import java.io.File;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import ch.kuramo.javie.app.widgets.WSWin32;
import ch.kuramo.javie.core.misc.Java2DFboBugFix;
import ch.kuramo.javie.core.misc.JsonicBugFix;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "ch.kuramo.javie.app";

	// The shared instance
	private static Activator plugin;


	private ColorRegistry colorRegistry;

	/**
	 * The constructor
	 */
	public Activator() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;

		// スペースが%20で表現されるURLでScannotationを使う場合は次の呼び出しを行ってScannotationのコードを修正する必要があるが、
		// FileLocator.resolveが返すURLはスペースが%20にならずスペースのままなので、オリジナルのScannotationでそのまま動作する。
		//ScannotationBugFix.patch();

		Java2DFboBugFix.patch();
		JsonicBugFix.patch();
		SplashEnhancement.patch();
		DragSourceEnhancement.patch();
		DropTargetEnhancement.patch();
		TreeViewerEditorEnhancement.patch();
		ColorDialogBugFix.patch();
		Java6Update26Detection.patch();


		File logFile = Platform.getLogFileLocation().toFile();
		if (logFile.exists()) {
			String newName = String.format("%1$s-%2$tY%2$tm%2$td-%2$tH%2$tM%2$tS", logFile.getName(), logFile.lastModified());
			logFile.renameTo(new File(logFile.getParentFile(), newName));
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}


	public static final String IMG_PLAYER_PLAY			= "IMG_PLAYER_PLAY";
	public static final String IMG_PLAYER_PAUSE			= "IMG_PLAYER_PAUSE";
	public static final String IMG_PLAYER_HEAD			= "IMG_PLAYER_HEAD";
	public static final String IMG_PLAYER_PREV			= "IMG_PLAYER_PREV";
	public static final String IMG_PLAYER_NEXT			= "IMG_PLAYER_NEXT";
	public static final String IMG_PLAYER_LAST			= "IMG_PLAYER_LAST";
	public static final String IMG_PLAYER_ZOOM			= "IMG_PLAYER_ZOOM";
	public static final String IMG_PLAYER_RESOLUTION	= "IMG_PLAYER_RESOLUTION";
	public static final String IMG_PLAYER_MUTE			= "IMG_PLAYER_MUTE";
	public static final String IMG_PLAYER_NOMUTE		= "IMG_PLAYER_NOMUTE";
	public static final String IMG_PLAYER_INFO			= "IMG_PLAYER_INFO";
	public static final String IMG_PLAYER_PIN			= "IMG_PLAYER_PIN";

	public static final String IMG_HEADER_SWITCHES				= "IMG_HEADER_SWITCHES";
	public static final String IMG_HEADER_SHOWHIDE				= "IMG_HEADER_SHOWHIDE";
	public static final String IMG_HEADER_SHOWHIDE_XP_THEMED	= "IMG_HEADER_SHOWHIDE_XP_THEMED";
	public static final String IMG_HEADER_SHOWHIDE_XP_CLASSIC	= "IMG_HEADER_SHOWHIDE_XP_CLASSIC";

	public static final String IMG_COMP_SETTINGS		= "IMG_COMP_SETTINGS";
	public static final String IMG_COMP_SS_NONE			= "IMG_COMP_SS_NONE";
	public static final String IMG_COMP_SS_2X2			= "IMG_COMP_SS_2X2";
	public static final String IMG_COMP_SS_3X3			= "IMG_COMP_SS_3X3";
	public static final String IMG_COMP_SS_4X4			= "IMG_COMP_SS_4X4";
	public static final String IMG_COMP_MBLUR			= "IMG_COMP_MBLUR";
	public static final String IMG_COMP_FBLEND			= "IMG_COMP_FBLEND";
	public static final String IMG_COMP_SHY				= "IMG_COMP_SHY";

	public static final String IMG_LIGHT	= "IMG_LIGHT";
	public static final String IMG_CAMERA	= "IMG_CAMERA";
	public static final String IMG_TEXT		= "IMG_TEXT";

	public static final String IMG_SWITCH_VIDEO = "IMG_SWITCH_VIDEO";
	public static final String IMG_SWITCH_AUDIO = "IMG_SWITCH_AUDIO";

	public static final String IMG_SWITCH_UNSHY				= "IMG_SWITCH_UNSHY";
	public static final String IMG_SWITCH_SHY				= "IMG_SWITCH_SHY";
	public static final String IMG_SWITCH_QUALITY_DRAFT		= "IMG_SWITCH_QUALITY_DRAFT";
	public static final String IMG_SWITCH_QUALITY_NORMAL	= "IMG_SWITCH_QUALITY_NORMAL";
	public static final String IMG_SWITCH_QUALITY_BEST		= "IMG_SWITCH_QUALITY_BEST";
	public static final String IMG_SWITCH_EFFECT			= "IMG_SWITCH_EFFECT";
	public static final String IMG_SWITCH_CTCR				= "IMG_SWITCH_CTCR";
	public static final String IMG_SWITCH_FBLEND			= "IMG_SWITCH_FBLEND";
	public static final String IMG_SWITCH_MBLUR_TRANSFORM	= "IMG_SWITCH_MBLUR_TRANSFORM";
	public static final String IMG_SWITCH_MBLUR_FULL		= "IMG_SWITCH_MBLUR_FULL";
	public static final String IMG_SWITCH_3D				= "IMG_SWITCH_3D";
	public static final String IMG_SWITCH_PERCHAR3D			= "IMG_SWITCH_PERCHAR3D";
	public static final String IMG_SWITCH_LINK				= "IMG_SWITCH_LINK";
	public static final String IMG_SWITCH_TINY_2D_MANIP		= "IMG_SWITCH_TINY_2D_MANIP";

	public static final String IMG_KF_HOLD					= "IMG_KF_HOLD";
	public static final String IMG_KF_HOLD_SELECT			= "IMG_KF_HOLD_SELECT";
	public static final String IMG_KF_LINEAR				= "IMG_KF_LINEAR";
	public static final String IMG_KF_LINEAR_SELECT			= "IMG_KF_LINEAR_SELECT";
	public static final String IMG_KF_CATMULL_ROM			= "IMG_KF_CATMULL_ROM";
	public static final String IMG_KF_CATMULL_ROM_SELECT	= "IMG_KF_CATMULL_ROM_SELECT";

	public static final String IMG_KFNAV_PREV				= "IMG_KFNAV_PREV";
	public static final String IMG_KFNAV_PREV_DISABLED		= "IMG_KFNAV_PREV_DISABLED";
	public static final String IMG_KFNAV_NEXT				= "IMG_KFNAV_NEXT";
	public static final String IMG_KFNAV_NEXT_DISABLED		= "IMG_KFNAV_NEXT_DISABLED";

	public static final String IMG_STOPWATCH_NO_KF	= "IMG_STOPWATCH_NO_KF";
	public static final String IMG_STOPWATCH_HAS_KF	= "IMG_STOPWATCH_HAS_KF";

	private static final boolean WIN32 = SWT.getPlatform().equals("win32");
	private static final boolean COCOA = SWT.getPlatform().equals("cocoa");

	protected void initializeImageRegistry(ImageRegistry reg) {
		registerImage(reg, IMG_PLAYER_PLAY, "player_play.png");
		registerImage(reg, IMG_PLAYER_PAUSE, "player_pause.png");
		registerImage(reg, IMG_PLAYER_HEAD, "player_head.png");
		registerImage(reg, IMG_PLAYER_PREV, "player_prev.png");
		registerImage(reg, IMG_PLAYER_NEXT, "player_next.png");
		registerImage(reg, IMG_PLAYER_LAST, "player_last.png");
		registerImage(reg, IMG_PLAYER_ZOOM, "player_zoom.png");
		registerImage(reg, IMG_PLAYER_RESOLUTION, "player_resolution.png");
		registerImage(reg, IMG_PLAYER_MUTE, "player_mute.png");
		registerImage(reg, IMG_PLAYER_NOMUTE, "player_nomute.png");
		registerImage(reg, IMG_PLAYER_INFO, "player_info.png");
		registerImage(reg, IMG_PLAYER_PIN, "player_pin.png");

		if (WSWin32.isWin32()) {
			registerImage(reg, IMG_HEADER_SWITCHES, "header_switches_win32.png");

			if (WSWin32.isWinXP()) {
				registerImage(reg, IMG_HEADER_SHOWHIDE_XP_THEMED, "header_showhide_xp_themed.png");
				registerImage(reg, IMG_HEADER_SHOWHIDE_XP_CLASSIC, "header_showhide_xp_classic.png");
			} else {
				registerImage(reg, IMG_HEADER_SHOWHIDE, "header_showhide_win32.png");
			}
		} else {
			registerImage(reg, IMG_HEADER_SWITCHES, "header_switches.png");
			registerImage(reg, IMG_HEADER_SHOWHIDE, "header_showhide.png");
		}

		registerImage(reg, IMG_COMP_SETTINGS, "comp_settings.png");
		registerImage(reg, IMG_COMP_SS_NONE, "comp_ss_none.png");
		registerImage(reg, IMG_COMP_SS_2X2, "comp_ss_2x2.png");
		registerImage(reg, IMG_COMP_SS_3X3, "comp_ss_3x3.png");
		registerImage(reg, IMG_COMP_SS_4X4, "comp_ss_4x4.png");
		registerImage(reg, IMG_COMP_MBLUR, "comp_mblur.png");
		registerImage(reg, IMG_COMP_FBLEND, "comp_fblend.png");
		registerImage(reg, IMG_COMP_SHY, "comp_shy.png");

		registerImage(reg, IMG_LIGHT, "light.png");
		registerImage(reg, IMG_CAMERA, "camera.png");
		registerImage(reg, IMG_TEXT, "text.png");

		registerImage(reg, IMG_SWITCH_VIDEO, "switch_video.png");
		registerImage(reg, IMG_SWITCH_AUDIO, "switch_audio.png");

		registerImage(reg, IMG_SWITCH_UNSHY, "switch_unshy.png");
		registerImage(reg, IMG_SWITCH_SHY, "switch_shy.png");
		registerImage(reg, IMG_SWITCH_CTCR, "switch_ctcr.png");
		registerImage(reg, IMG_SWITCH_QUALITY_DRAFT, "switch_quality_draft.png");
		registerImage(reg, IMG_SWITCH_QUALITY_NORMAL, "switch_quality_normal.png");
		registerImage(reg, IMG_SWITCH_QUALITY_BEST, "switch_quality_best.png");
		registerImage(reg, IMG_SWITCH_FBLEND, "switch_fblend.png");
		registerImage(reg, IMG_SWITCH_MBLUR_TRANSFORM, "switch_mblur_transform.png");
		registerImage(reg, IMG_SWITCH_MBLUR_FULL, "switch_mblur_full.png");
		registerImage(reg, IMG_SWITCH_EFFECT, "switch_effect.png");
		registerImage(reg, IMG_SWITCH_3D, "switch_3d.png");
		registerImage(reg, IMG_SWITCH_PERCHAR3D, "switch_perchar3d.png");
		registerImage(reg, IMG_SWITCH_LINK, "switch_link.png");
		registerImage(reg, IMG_SWITCH_TINY_2D_MANIP, "switch_tiny_2d_manip.png");

		if (WIN32) {
			registerImage(reg, IMG_KF_HOLD, "kf_hold_win32.png");
			registerImage(reg, IMG_KF_LINEAR, "kf_linear_win32.png");
			registerImage(reg, IMG_KF_CATMULL_ROM, "kf_catmull_rom_win32.png");
		} else if (COCOA) {
			registerImage(reg, IMG_KF_HOLD,        "kf_hold_cocoa.png");
			registerImage(reg, IMG_KF_HOLD_SELECT, "kf_hold_select_cocoa.png");
			registerImage(reg, IMG_KF_LINEAR,        "kf_linear_cocoa.png");
			registerImage(reg, IMG_KF_LINEAR_SELECT, "kf_linear_select_cocoa.png");
			registerImage(reg, IMG_KF_CATMULL_ROM,        "kf_catmull_rom_cocoa.png");
			registerImage(reg, IMG_KF_CATMULL_ROM_SELECT, "kf_catmull_rom_select_cocoa.png");
		} else {
			// FIXME
		}

		registerImage(reg, IMG_KFNAV_PREV,          "kfnav_prev.png");
		registerImage(reg, IMG_KFNAV_PREV_DISABLED, "kfnav_prev_disabled.png");
		registerImage(reg, IMG_KFNAV_NEXT,          "kfnav_next.png");
		registerImage(reg, IMG_KFNAV_NEXT_DISABLED, "kfnav_next_disabled.png");

		registerImage(reg, IMG_STOPWATCH_NO_KF,  "stopwatch_no_kf.png");
		registerImage(reg, IMG_STOPWATCH_HAS_KF, "stopwatch_has_kf.png");
	}

	private void registerImage(ImageRegistry registry, String key, String filename) {
		IPath path = new Path("icons/" + filename);
		URL url = FileLocator.find(getBundle(), path, null);
		if (url != null) {
			ImageDescriptor desc = ImageDescriptor.createFromURL(url);
			registry.put(key, desc);
		}
	}


	public static final String COLOR_TABLE_BACKGROUND		= "COLOR_TABLE_BACKGROUND";
	public static final String COLOR_TABLE_ROW_LINE_COCOA	= "COLOR_TABLE_ROW_LINE_COCOA";
	public static final String COLOR_LAYER_ROW_BACKGROUND	= "COLOR_LAYER_ROW_BACKGROUND";

	public static final String COLOR_SWITCH_BORDER_COCOA_BLUE_TL	= "COLOR_SWITCH_BORDER_COCOA_BLUE_TL";
	public static final String COLOR_SWITCH_BORDER_COCOA_BLUE_BR	= "COLOR_SWITCH_BORDER_COCOA_BLUE_BR";
	public static final String COLOR_SWITCH_BORDER_COCOA_GRAY_TL	= "COLOR_SWITCH_BORDER_COCOA_GRAY_TL";
	public static final String COLOR_SWITCH_BORDER_COCOA_GRAY_BR	= "COLOR_SWITCH_BORDER_COCOA_GRAY_BR";
	public static final String COLOR_SWITCH_BORDER_LIGHT_GRAY_TL	= "COLOR_SWITCH_BORDER_LIGHT_GRAY_TL";
	public static final String COLOR_SWITCH_BORDER_LIGHT_GRAY_BR	= "COLOR_SWITCH_BORDER_LIGHT_GRAY_BR";

	public static final String COLOR_LAYERBAR			= "COLOR_LAYERBAR";
	public static final String COLOR_LAYER_EXCEEDED		= "COLOR_LAYER_EXCEEDED";
	public static final String COLOR_LAYER_SLIPBAR		= "COLOR_LAYER_SLIPBAR";

	public static final String COLOR_KEYFRAME			= "COLOR_KEYFRAME";
	public static final String COLOR_KEYFRAME_SELECTED	= "COLOR_KEYFRAME_SELECTED";
	public static final String COLOR_KEYFRAME_BORDER	= "COLOR_KEYFRAME_BORDER";


	public ColorRegistry getColorRegistry() {
		if (colorRegistry == null) {
			colorRegistry = createColorRegistry();
			initializeColorRegistry(colorRegistry);
		}
		return colorRegistry;
	}

	protected ColorRegistry createColorRegistry() {
		// AbstractUIPlugin#createImageRegistry と同じように ColorRegistry を作成する。

		if(Display.getCurrent() != null) {
			return new ColorRegistry(Display.getCurrent());
		}

		if(PlatformUI.isWorkbenchRunning()) {
			return new ColorRegistry(PlatformUI.getWorkbench().getDisplay());
		}

		throw new SWTError(SWT.ERROR_THREAD_INVALID_ACCESS);
	}

	protected void initializeColorRegistry(ColorRegistry reg) {
		reg.put(COLOR_TABLE_BACKGROUND, new RGB(192, 192, 192));
		reg.put(COLOR_TABLE_ROW_LINE_COCOA, new RGB(232, 232, 232));
		reg.put(COLOR_LAYER_ROW_BACKGROUND, new RGB(168, 168, 168));

		reg.put(COLOR_SWITCH_BORDER_COCOA_BLUE_TL, new RGB(47, 90, 145));
		reg.put(COLOR_SWITCH_BORDER_COCOA_BLUE_BR, new RGB(74, 142, 230));
		reg.put(COLOR_SWITCH_BORDER_COCOA_GRAY_TL, new RGB(153, 153, 153));
		reg.put(COLOR_SWITCH_BORDER_COCOA_GRAY_BR, new RGB(240, 240, 240));
		reg.put(COLOR_SWITCH_BORDER_LIGHT_GRAY_TL, new RGB(140, 140, 140));
		reg.put(COLOR_SWITCH_BORDER_LIGHT_GRAY_BR, new RGB(212, 212, 212));

		reg.put(COLOR_LAYERBAR, new RGB(158, 62, 62));
		reg.put(COLOR_LAYER_EXCEEDED, new RGB(147, 83, 83));
		reg.put(COLOR_LAYER_SLIPBAR, new RGB(189, 125, 125));

		reg.put(COLOR_KEYFRAME, new RGB(161, 161, 161));
		reg.put(COLOR_KEYFRAME_SELECTED, new RGB(255, 205, 71));
		reg.put(COLOR_KEYFRAME_BORDER, new RGB(36, 36, 36));
	}

}
