/*
 * Copyright (c) 2009,2010 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.core.internal.services;

import java.util.LinkedList;

import ch.kuramo.javie.api.IAnimatableValue;
import ch.kuramo.javie.api.IAudioBuffer;
import ch.kuramo.javie.api.IObjectArray;
import ch.kuramo.javie.api.services.IAudioEffectContext;
import ch.kuramo.javie.core.Util;

public class AudioEffectContextProxy
		extends AbstractEffectContextProxy<IAudioEffectContext>
		implements IAudioEffectContext {

	private static final ThreadLocal<LinkedList<IAudioEffectContext>>
			_threadLocal = new ThreadLocal<LinkedList<IAudioEffectContext>>();

	static void push(IAudioEffectContext impl) {
		getStack(true).addLast(impl);
	}

	static void pop() {
		LinkedList<IAudioEffectContext> stack = getStack(false);

		stack.removeLast();

		if (stack.isEmpty()) {
			_threadLocal.remove();
		}
	}

	private static LinkedList<IAudioEffectContext> getStack(boolean create) {
		LinkedList<IAudioEffectContext> stack = _threadLocal.get();
		if (stack == null) {
			if (create) {
				stack = Util.newLinkedList();
				_threadLocal.set(stack);
			} else {
				throw new IllegalStateException("no stack is found");
			}
		} else if (stack.isEmpty()) {
			// popメソッドにおいて、スタックが空になったときにはスレッドローカルからスタックを取り除いているので、
			// ここで空のスタックが返されることは無い。
			throw new IllegalStateException("stack is empty");
		}
		return stack;
	}

	protected IAudioEffectContext getImpl() {
		return getStack(false).getLast();
	}

	public <V> IObjectArray<V> values(IAnimatableValue<V> avalue) {
		return getImpl().values(avalue);
	}

	@Deprecated
	public <V> IObjectArray<V> values(IAnimatableValue<V> avalue, int animationRate) {
		return getImpl().values(avalue, animationRate);
	}

	public IAudioBuffer doPreviousEffect() {
		return getImpl().doPreviousEffect();
	}

}
