/*
 * Copyright (c) 2009-2011 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.core.services;

import java.io.File;

import org.scannotation.AnnotationDB;

import ch.kuramo.javie.api.Color;
import ch.kuramo.javie.api.ColorMode;
import ch.kuramo.javie.api.Size2i;
import ch.kuramo.javie.api.Time;
import ch.kuramo.javie.core.CameraLayer;
import ch.kuramo.javie.core.Composition;
import ch.kuramo.javie.core.CompositionItem;
import ch.kuramo.javie.core.FileItem;
import ch.kuramo.javie.core.Folder;
import ch.kuramo.javie.core.ImageSequenceItem;
import ch.kuramo.javie.core.LayerComposition;
import ch.kuramo.javie.core.LightLayer;
import ch.kuramo.javie.core.MediaItem;
import ch.kuramo.javie.core.MediaItemLayer;
import ch.kuramo.javie.core.NullLayer;
import ch.kuramo.javie.core.Project;
import ch.kuramo.javie.core.SolidColorItem;
import ch.kuramo.javie.core.TARangeSelector;
import ch.kuramo.javie.core.TAWigglySelector;
import ch.kuramo.javie.core.TextAnimator;
import ch.kuramo.javie.core.TextLayer;

public interface ProjectElementFactory {

	void searchClasses(AnnotationDB db, ClassLoader cl);

	<T> Class<T> getClass(String type);


	Project newProject();

	Folder newFolder();

	FileItem newFileItem(File file);

	ImageSequenceItem newImageSequenceItem(File file, Time frameDuration);

	SolidColorItem newSolidColorItem(Color color, Size2i size);

	CompositionItem newCompositionItem(Composition composition);

	LayerComposition newLayerComposition(
			ColorMode colorMode, Size2i size, Time frameDuration, Time duration);

	LightLayer newLightLayer();

	CameraLayer newCameraLayer();

	MediaItemLayer newMediaItemLayer(MediaItem mediaItem);

	NullLayer newNullLayer();

	TextLayer newTextLayer();

	TextAnimator newTextAnimator();

	TARangeSelector newTARangeSelector();

	TAWigglySelector newTAWigglySelector();

}
